@@ -19,6 +19,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef mscommon_h
 #define mscommon_h
 
+#include <bctoolbox/port.h>
+#include <bctoolbox/list.h>
 #include <ortp/logging.h>
 #include <ortp/port.h>
 #include <ortp/str_utils.h>
@@ -28,6 +30,13 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "TargetConditionals.h"
 #endif
 
+#ifndef MS2_DEPRECATED
+#if defined(_MSC_VER)
+#define MS2_DEPRECATED __declspec(deprecated)
+#else
+#define MS2_DEPRECATED __attribute__ ((deprecated))
+#endif
+#endif
 #define MS_UNUSED(x) ((void)(x))
 
 #define ms_malloc	ortp_malloc
@@ -54,6 +63,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define ms_cond_broadcast	ortp_cond_broadcast
 #define ms_cond_destroy		ortp_cond_destroy
 
+#define MS_DEFAULT_MAX_PAYLOAD_SIZE 1440
+
 #define MS2_INLINE ORTP_INLINE
 
 #ifdef _WIN32
@@ -89,7 +100,7 @@ static MS2_INLINE void ms_debug(const char *fmt,...)
 {
   va_list args;
   va_start (args, fmt);
-  ortp_logv(ORTP_DEBUG, fmt, args);
+  ortp_logv(ORTP_LOG_DOMAIN, ORTP_DEBUG, fmt, args);
   va_end (args);
 }
 #else
@@ -117,30 +128,23 @@ typedef ortpTimeSpec MSTimeSpec;
 #define ms_get_cur_time ortp_get_cur_time
 #define ms_get_cur_time_ms ortp_get_cur_time_ms
 
-struct _MSList {
-	struct _MSList *next;
-	struct _MSList *prev;
-	void *data;
-};
-
-typedef struct _MSList MSList;
-
-
-#define ms_list_next(elem) ((elem)->next)
-
-
-typedef int (*MSCompareFunc)(const void *a, const void *b);
+typedef bctbx_compare_func MSCompareFunc;
 typedef void (*MSIterateFunc)(void *a);
 typedef void (*MSIterate2Func)(void *a, void *b);
+typedef void (*MSIterate3Func)(void *a, void *b, void *c);
 
 #ifdef __cplusplus
 extern "C"{
 #endif
 /*for stun*/
-typedef struct { unsigned char octet[12]; }  UInt96;
+typedef struct { unsigned char octet[12]; } UInt96;
+typedef struct { unsigned char octet[16]; } UInt128;
 
 MS2_PUBLIC void ms_thread_exit(void* ret_val);
 
+
+#define MSList bctbx_list_t
+
 /**
  * @addtogroup ms_list
  * @{
@@ -150,150 +154,178 @@ MS2_PUBLIC void ms_thread_exit(void* ret_val);
  * @param list list where data should be added. If NULL, a new list will be created.
  * @param data data to insert into the list
  * @return first element of the list
+ * @deprecated Use bctbx_list_append() instead
 **/
-MS2_PUBLIC MSList * ms_list_append(MSList *list, void * data);
+MS2_PUBLIC MS2_DEPRECATED MSList * ms_list_append(MSList *list, void * data);
 
 /** Inserts given element to the end of a given list
  * @param list list where data should be added. If NULL, a new list will be created.
  * @param new_elem element to append
  * @return first element of the list
+ * @deprecated Use bctbx_list_append_link() instead
 **/
-MS2_PUBLIC MSList *ms_list_append_link(MSList *list, MSList *new_elem);
+MS2_PUBLIC MS2_DEPRECATED MSList *ms_list_append_link(MSList *list, MSList *new_elem);
 
 /** Inserts a new element containing data to the start of a given list
  * @param list list where data should be added. If NULL, a new list will be created.
  * @param data data to insert into the list
  * @return first element of the list - the one which was just created.
+ * @deprecated Use bctbx_list_prepend() instead
 **/
-MS2_PUBLIC MSList * ms_list_prepend(MSList *list, void * data);
+MS2_PUBLIC MS2_DEPRECATED MSList * ms_list_prepend(MSList *list, void * data);
 
 /** Frees all elements of a given list
  * Note that data contained in each element will not be freed. If you need to clean
  * them, consider using @ms_list_free_with_data
  * @param list object to free.
  * @return NULL
+ * @deprecated Use bctbx_list_free() instead
 **/
-MS2_PUBLIC MSList * ms_list_free(MSList *list);
+MS2_PUBLIC MS2_DEPRECATED MSList * ms_list_free(MSList *list);
 
 /** Frees all elements of a given list after having called freefunc on each element
  * @param list object to free.
  * @param freefunc function to invoke on each element data before destroying the element
  * @return NULL
+ * @deprecated Use bctbx_list_free_with_data() instead
 **/
-MS2_PUBLIC MSList * ms_list_free_with_data(MSList *list, void (*freefunc)(void*));
+MS2_PUBLIC MS2_DEPRECATED MSList * ms_list_free_with_data(MSList *list, void (*freefunc)(void*));
 
 /** Concatenates second list to the end of first list
  * @param first First list
  * @param second Second list to append at the end of first list.
  * @return first element of the merged list
+ * @deprecated Use bctbx_list_concat() instead
 **/
-MS2_PUBLIC MSList * ms_list_concat(MSList *first, MSList *second);
+MS2_PUBLIC MS2_DEPRECATED MSList * ms_list_concat(MSList *first, MSList *second);
 
 /** Finds and remove the first element containing the given data. Nothing is done if element is not found.
  * @param list List in which data must be removed
  * @param data Data to remove
  * @return first element of the modified list
+ * @deprecated Use bctbx_list_remove() instead
 **/
-MS2_PUBLIC MSList * ms_list_remove(MSList *list, void *data);
+MS2_PUBLIC MS2_DEPRECATED MSList * ms_list_remove(MSList *list, void *data);
 
 /** Finds and remove any elements according to the given predicate function
  * @param list List in which data must be removed
  * @param compare_func Function to invoke on each element. If it returns TRUE, the given element will be deleted.
  * @param user_data User data to pass to compare_func function
  * @return first element of the modified list
+ * @deprecated Use bctbx_list_remove_custom() instead
 **/
-MS2_PUBLIC MSList * ms_list_remove_custom(MSList *list, MSCompareFunc compare_func, const void *user_data);
+MS2_PUBLIC MS2_DEPRECATED MSList * ms_list_remove_custom(MSList *list, MSCompareFunc compare_func, const void *user_data);
 
 /** Returns size of a given list
  * @param list List to measure
  * @return Size of list
+ * @deprecated Use bctbx_list_size() instead
 **/
-MS2_PUBLIC int ms_list_size(const MSList *list);
+MS2_PUBLIC MS2_DEPRECATED int ms_list_size(const MSList *list);
 
 /** Invoke function on each element of the list
  * @param list List object
  * @param iterate_func Function to invoke on each element.
+ * @deprecated Use bctbx_list_for_each() instead
 **/
-MS2_PUBLIC void ms_list_for_each(const MSList *list, MSIterateFunc iterate_func);
+MS2_PUBLIC MS2_DEPRECATED void ms_list_for_each(const MSList *list, MSIterateFunc iterate_func);
 
 /** Invoke function on each element of the list
  * @param list List object
  * @param iterate_func Function to invoke on each element.
  * @param user_data User data to pass to iterate_func function.
+ * @deprecated Use bctbx_list_for_each2() instead
 **/
-MS2_PUBLIC void ms_list_for_each2(const MSList *list, MSIterate2Func iterate_func, void *user_data);
+MS2_PUBLIC MS2_DEPRECATED void ms_list_for_each2(const MSList *list, MSIterate2Func iterate_func, void *user_data);
+	
+MS2_PUBLIC MS2_DEPRECATED void ms_list_for_each3(const MSList *list, MSIterate3Func iterate_func, void *user_data, void *factory);
 
 /** Finds and remove given element in list.
  * @param list List in which element must be removed
  * @param element element to remove
  * @return first element of the modified list
+ * @deprecated Use bctbx_list_remove_link() instead
 **/
-MS2_PUBLIC MSList *ms_list_remove_link(MSList *list, MSList *elem);
+MS2_PUBLIC MS2_DEPRECATED MSList *ms_list_remove_link(MSList *list, MSList *elem);
 
 /** Finds first element containing data in the given list.
  * @param list List in which element must be found
  * @param data data to find
  * @return element containing data, or NULL if not found
+ * @deprecated Use bctbx_list_find() instead
 **/
-MS2_PUBLIC MSList *ms_list_find(MSList *list, void *data);
+MS2_PUBLIC MS2_DEPRECATED MSList *ms_list_find(MSList *list, void *data);
 
 /** Finds first element according to the given predicate function
  * @param list List in which element must be found
  * @param compare_func Function to invoke on each element. If it returns TRUE, the given element will be returned.
  * @param user_data User data to pass to compare_func function
  * @return Element matching the predicate, or NULL if none is found.
+ * @deprecated Use bctbx_list_find_custom() instead
 **/
-MS2_PUBLIC MSList *ms_list_find_custom(MSList *list, MSCompareFunc compare_func, const void *user_data);
+MS2_PUBLIC MS2_DEPRECATED MSList *ms_list_find_custom(MSList *list, MSCompareFunc compare_func, const void *user_data);
 
 /** Returns the nth element data of the list
  * @param list List object
  * @param index data index which must be returned.
  * @return Element at the given index. NULL if index is invalid (negative or greater or equal to ms_list_size).
+ * @deprecated Use bctbx_list_nth_data() instead
 **/
-MS2_PUBLIC void * ms_list_nth_data(const MSList *list, int index);
+MS2_PUBLIC MS2_DEPRECATED void * ms_list_nth_data(const MSList *list, int index);
 
 /** Returns the index of the given element
  * @param list List object
  * @param elem Element to search for.
  * @return Index of the given element. -1 if not found.
+ * @deprecated Use bctbx_list_position() instead
 **/
-MS2_PUBLIC int ms_list_position(const MSList *list, MSList *elem);
+MS2_PUBLIC MS2_DEPRECATED int ms_list_position(const MSList *list, MSList *elem);
 
 /** Returns the index of the first element containing data
  * @param list List object
  * @param data Data to search for.
  * @return Index of the element containing data. -1 if not found.
+ * @deprecated Use bctbx_list_index() instead
 **/
-MS2_PUBLIC int ms_list_index(const MSList *list, void *data);
+MS2_PUBLIC MS2_DEPRECATED int ms_list_index(const MSList *list, void *data);
 
 /** Inserts a new element containing data at the place given by compare_func
  * @param list list where data should be added. If NULL, a new list will be created.
  * @param data data to insert into the list
  * @param compare_func function determining where should the new element be placed
  * @return first element of the list.
+ * @deprecated Use bctbx_list_insert_sorted() instead
 **/
-MS2_PUBLIC MSList *ms_list_insert_sorted(MSList *list, void *data, MSCompareFunc compare_func);
+MS2_PUBLIC MS2_DEPRECATED MSList *ms_list_insert_sorted(MSList *list, void *data, MSCompareFunc compare_func);
 
 /** Inserts a new element containing data before the given element
  * @param list list where data should be added. If NULL, a new list will be created.
  * @param before element parent to the one we will insert.
  * @param data data to insert into the list
  * @return first element of the modified list.
+ * @deprecated Use bctbx_list_insert() instead
 **/
-MS2_PUBLIC MSList *ms_list_insert(MSList *list, MSList *before, void *data);
+MS2_PUBLIC MS2_DEPRECATED MSList *ms_list_insert(MSList *list, MSList *before, void *data);
 
 /** Copies a list in another one, duplicating elements but not data
  * @param list list to copy
  * @return Newly created list
+ * @deprecated Use bctbx_list_copy() instead
 **/
-MS2_PUBLIC MSList *ms_list_copy(const MSList *list);
+MS2_PUBLIC MS2_DEPRECATED MSList *ms_list_copy(const MSList *list);
 
 /** Copies a list in another one, duplicating elements according to the given function
  * @param list list to copy
  * @param copyfunc function to invoke on each element which will return the new list data value
  * @return Newly created list
+ * @deprecated Use bctbx_list_copy_with_data() instead
 **/
-MS2_PUBLIC MSList *ms_list_copy_with_data(const MSList *list, void *(*copyfunc)(void *));
+MS2_PUBLIC MS2_DEPRECATED MSList *ms_list_copy_with_data(const MSList *list, void *(*copyfunc)(void *));
+
+/**
+ * @deprecated @deprecated Use bctbx_list_next() instead
+ */
+MS2_PUBLIC MS2_DEPRECATED MSList* ms_list_next(const MSList *list);
 
 /** @} */
 
@@ -336,29 +368,33 @@ MS2_PUBLIC bool_t ms_tags_list_contains_tag(const MSList *list, const char *tag)
  * Initialize the mediastreamer2 base library.
  *
  * This must be called once before calling any other API.
+ * @deprecated use ms_factory_new()
  */
-MS2_PUBLIC void ms_base_init(void);
+MS2_PUBLIC MS2_DEPRECATED void ms_base_init(void);
 
 /**
  * Initialize the mediastreamer2 VoIP library.
  *
  * This must be called one before calling any other API.
+ * @deprecated use ms_factory_new_with_voip().
  */
-MS2_PUBLIC void ms_voip_init(void);
+MS2_PUBLIC MS2_DEPRECATED void ms_voip_init(void);
 
 /**
  * Load the plugins from the default plugin directory.
  *
  * This is just a wrapper around ms_load_plugins().
  * This must be called after ms_base_init() and after ms_voip_init().
+ * @deprecated use ms_factory_init_plugins(), or ms_factory_new_with_voip() that does it automatically.
  */
-MS2_PUBLIC void ms_plugins_init(void);
+MS2_PUBLIC MS2_DEPRECATED void ms_plugins_init(void);
 
 /**
  * Set the directory from where the plugins are to be loaded when calling ms_plugins_init().
  * @param[in] path The path to the plugins directory.
+ * @deprecated use ms_factory_set_plugins_dir().
  */
-MS2_PUBLIC void ms_set_plugins_dir(const char *path);
+MS2_PUBLIC MS2_DEPRECATED void ms_set_plugins_dir(const char *path);
 
 /**
  * Load plugins from a specific directory.
@@ -369,28 +405,32 @@ MS2_PUBLIC void ms_set_plugins_dir(const char *path);
  *
  * @param directory   A directory where plugins library are available.
  *
- * Returns: >0 if successfull, 0 if not plugins loaded, -1 otherwise.
+ * @return >0 if successfull, 0 if not plugins loaded, -1 otherwise.
+ * @deprecated use ms_factory_load_plugins().
  */
-MS2_PUBLIC int ms_load_plugins(const char *directory);
+MS2_PUBLIC MS2_DEPRECATED int ms_load_plugins(const char *directory);
 
 /**
  * Release resource allocated in the mediastreamer2 base library.
  *
  * This must be called once before closing program.
+ * @deprecated use ms_factory_destroy().
  */
-MS2_PUBLIC void ms_base_exit(void);
+MS2_PUBLIC MS2_DEPRECATED void  ms_base_exit(void);
 
 /**
  * Release resource allocated in the mediastreamer2 VoIP library.
  *
  * This must be called once before closing program.
+ * @deprecated use ms_factory_destroy().
  */
-MS2_PUBLIC void ms_voip_exit(void);
+MS2_PUBLIC MS2_DEPRECATED void ms_voip_exit(void);
 
 /**
  * Unload the plugins loaded by ms_plugins_init().
+ * @deprecated use ms_factory_destroy().
  */
-MS2_PUBLIC void ms_plugins_exit(void);
+MS2_PUBLIC MS2_DEPRECATED void ms_plugins_exit(void);
 
 struct _MSSndCardDesc;
 
@@ -405,11 +445,15 @@ MS2_PUBLIC void ms_usleep(uint64_t usec);
  * The default value is 1440 computed as the standard internet MTU minus IPv6 header,
  * UDP header and RTP header. As IPV4 header is smaller than IPv6 header, this
  * value works for both.
- *
+ * @deprecated use ms_factory_get_payload_max_size().
 **/
-MS2_PUBLIC int ms_get_payload_max_size(void);
+MS2_PUBLIC MS2_DEPRECATED int ms_get_payload_max_size(void);
 
-MS2_PUBLIC void ms_set_payload_max_size(int size);
+/**
+ * Set the maximum payload size allowed.
+ * @deprecated use ms_factory_set_payload_max_size().
+**/
+MS2_PUBLIC MS2_DEPRECATED void ms_set_payload_max_size(int size);
 
 /**
  * Returns the network Max Transmission Unit to reach destination_host.
@@ -421,21 +465,26 @@ MS2_PUBLIC int ms_discover_mtu(const char *destination_host);
 /**
  * Set mediastreamer default mtu, used to compute the default RTP max payload size.
  * This function will call ms_set_payload_max_size(mtu-[ipv6 header size]).
+ * @deprecated use ms_factory_set_mtu()
 **/
-MS2_PUBLIC void ms_set_mtu(int mtu);
-
+MS2_PUBLIC MS2_DEPRECATED void ms_set_mtu(int mtu);
 
 /**
  * Get mediastreamer default mtu, used to compute the default RTP max payload size.
+ * @deprecated use ms_factory_get_mtu().
 **/
-MS2_PUBLIC int ms_get_mtu(void);
+MS2_PUBLIC MS2_DEPRECATED int ms_get_mtu(void);
 
 /**
  * Declare how many cpu (cores) are available on the platform
+ * @deprecated use ms_factory_set_cpu_count().
  */
-MS2_PUBLIC void ms_set_cpu_count(unsigned int c);
+MS2_PUBLIC MS2_DEPRECATED void ms_set_cpu_count(unsigned int c);
 
-MS2_PUBLIC unsigned int ms_get_cpu_count(void);
+/**
+ * @deprecated use ms_factory_get_cpu_count().
+**/
+MS2_PUBLIC MS2_DEPRECATED unsigned int ms_get_cpu_count(void);
 
 /**
  * Adds a new entry in the SoundDeviceDescription table
